<?php

namespace Drupal\entityreference_extensions\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceIdFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entityreference_extensions\EntityReferenceDeltaFilterTrait;

/**
 * Plugin implementation of the 'entity reference ID delta' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_entity_id_delta",
 *   label = @Translation("Entity ID (delta)"),
 *   description = @Translation("Display the ID of the referenced entities. Limit multi value field by delta."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceIdDeltaFormatter extends EntityReferenceIdFormatter {

  use EntityReferenceDeltaFilterTrait;

  /**
   * The number of items for current entity.
   *
   * @var int
   */
  protected $noItems;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'delta' => 'all',
      'reverse' => FALSE,
      'offset' => 0,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    if ($this->fieldCardinalityValid()) {
      $elements['delta'] = $this->deltaSetting();
      $elements['reverse'] = $this->reverseSetting();
      $elements['offset'] = $this->offsetSetting();
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    if ($this->fieldCardinalityValid()) {
      $summary[] = $this->getSetting('delta') == 'all' ? t('Show all') : t('Show @item', ['@item' => $this->getSetting('delta') + 1]);
      $summary[] = $this->getSetting('reverse') ? t('Reverse order') : '';
      $summary[] = $this->getSetting('offset') == 0 ? t('No offset') : t('Offset @item', ['@item' => $this->getSetting('offset')]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    if ($this->getSetting('delta') != 'all' && $this->fieldCardinalityValid()) {
      $this->noItems = count($items);
      $items->filter([$this, 'deltaFilter']);
    }
    $elements = parent::viewElements($items, $langcode);
    return $elements;
  }

}
